/* 
 * File:   ClipperMap.cpp
 * Author: tg
 * 
 * Created on July 19, 2012, 11:32 AM
 */

#include "ClipperMap.h"
#include <clipper/clipper-ccp4.h>
#include <iostream>

ClipperMap::ClipperMap(const std::vector<Reflex>& reflections, 
            const clipper::Spacegroup& sg,
            const clipper::Cell& cell,
        double dhighres)
: sg_(sg), cell_(cell), dhighres_(dhighres) {
    std::vector<Reflex>::const_iterator it;
    std::vector<clipper::HKL> hkls;

    std::cout << " ---> Created map from " 
            << reflections.size() << " reflections\n"
            << "        spacegroup "
            << sg_.symbol_hm()
            << "\n        cell "
            << cell_.format()
            << "\n";
    hklinfo_ = clipper::HKL_info(sg_, cell_, clipper::Resolution(dhighres_));


    for (it = reflections.begin(); it != reflections.end(); ++it) {
        clipper::HKL hkl(it->hkl().h(), it->hkl().k(), it->hkl().l());

        hkls.push_back(hkl);
    }
    hklinfo_.add_hkl_list(hkls);
    
    hkldataFphi_ = clipper::HKL_data<clipper::data64::F_phi > (hklinfo_, cell);

    for (it = reflections.begin(); it != reflections.end(); ++it) {
        clipper::HKL hkl(it->hkl().h(), it->hkl().k(), it->hkl().l());
        clipper::xtype fphi [2];
        fphi[0] = it->F();
        fphi[1] = M_PI/180.0 * it->phi();

        hkldataFphi_.data_import(hkl, fphi);
    }
}

void ClipperMap::makeMap() {
    clipper::Grid_sampling gs (sg_, cell_, hklinfo_.resolution());
    map_.init(sg_, cell_, gs);
    map_.fft_from(hkldataFphi_);
}

void ClipperMap::writeCCP4map(const char* filename) const {
    clipper::CCP4MAPfile mapfile;
    
    mapfile.open_write(filename);
    mapfile.export_xmap(map_);
    mapfile.close_write();
    
    std::cout << " ---> Map data written to \"" << filename << "\"\n";
}

