/* 
 * File:   Options.cpp
 * Author: tg
 * 
 * Created on July 23, 2012, 10:35 AM
 */

#include "Options.h"
#include "myExcepts.h"
#include <getopt.h>
#include <iostream>
#include <sstream>

void usage();

Options::Options(int argc, char** argv): m_(-1), w_ (-1.0), help_ (false), fom_(true) {
    int opt;
    std::istringstream inp;
    
    while ( (opt = getopt(argc, argv, "?hfm:o:s:w:")) != -1) {
        switch (opt) {
            case '?': case 'h':
                usage();
                help_ = true;
                return;
            case 'f':
                fom_ = false;
                break;
            case 'o':
                outputfn_ = optarg;
                break;
            case 'm':
                inp.clear();
                inp.str (optarg);
                inp >> m_;
                if (inp.fail()) {
                    usage();
                    std::cout << "*** Error: Unable to convert \""
                            << optarg 
                            << "\" to integer number\n";
                    throw myExcepts::Conversion::Option (optarg);
                }
                if (m_ < 0) {
                    std::cout << "*** Warning: m provided as negative number. "
                            << "Ignored and continuing as though m unset:\n"
                            << "    Creating 3Fo-2Fc and Fo-Fc maps\n";
                    m_ = -1;
                }
                break;
            case 's':
                pdbfile_ = optarg;
                break;
            case 'w':
                inp.str (optarg);
                inp >> w_;
                if (inp.fail()) {
                    usage();
                    std::cout << "*** Error: Unable to convert \""
                            << optarg
                            << "\" to floating point number\n";
                    throw myExcepts::Conversion::Option (optarg);
                }
                break;
            default:
                usage();
                throw myExcepts::Conversion::Option (optarg);
                return;
        }
    }
    /* get input file name */
    if (optind >= argc) {
        usage();
        std::cout << "\n*** Error: Please provide an input file\n";
        throw myExcepts::Conversion::Option ("Missing input file (.phs or .fcf)");
    }
    
    inputfn_ = argv[optind];
    ++optind;

    /* last remaining option for fa_file */
    if (optind < argc) {
        fafn_ = argv[optind];
    }
}


Options::~Options() {
}
