#include "ReflUtils.h"
#include "Vec3.h"
#include "conversions.h"
#include "myExcepts.h"

#include <complex>
#include <stdexcept>

/**
 * assuiming identical HKL indices this function returns the mean F and
 * phi for this reflection.
 * 
 * @param reflections
 * @return 
 */
Reflex ReflUtils::merge(const std::vector<Reflex>& reflections) {
    std::vector<Reflex>::const_iterator it;
    std::complex<double> mean_sf (0,0);
    double F;
    double phi;
    
    for (it=reflections.begin(); it != reflections.end(); ++it) {
        double angle = M_PI / 180.0 * it->phi();
        double real = it->F() * std::cos(angle);
        double img  = it->F() * std::sin(angle);
        
        std::complex<double> sf (real, img);
        mean_sf += sf;
    }
    mean_sf *= 1.0/reflections.size();
    
    F = std::abs(mean_sf);
    phi = std::arg(mean_sf);
    phi *= 180.0/M_PI;
    
    HKL hkl (reflections.front().hkl());
    return Reflex(hkl, F, phi);
}

double ReflUtils::dmin(const std::vector<Reflex>& reflections, const clipper::Cell& cell) {
    Vec3 a,b,c;
    Vec3 astar, bstar, cstar;
    double invdmaxsq(-1.0);
    std::vector<Reflex>::const_iterator it;
    
    cellVectors(cell.a(), cell.b(), cell.c(), 
            cell.alpha(), cell.beta(), cell.gamma(), 
            a, b, c);
    recCell(a, b, c, astar, bstar, cstar);
    
    for (it=reflections.begin(); it != reflections.end(); ++it) {
        double invdsq = it->dstarsq(astar, bstar, cstar);
        invdmaxsq = std::max(invdmaxsq, invdsq);
    }
    
    if (invdmaxsq < 0.0) {
        std::cout << "*** Error: invalid resolution\n";
        throw myExcepts::Conceptual("Invalid resolution.");
    }
    
    invdmaxsq = std::sqrt (invdmaxsq);
    invdmaxsq = 1.0/invdmaxsq;
    
    return invdmaxsq;
}
