/* 
 * File:   fcffile.h
 * Author: tg
 *
 * Created on July 9, 2012, 8:05 PM
 */

#ifndef FCFFILE_H
#define	FCFFILE_H

#include "mapcoeffile.h"
#include "Reflex.h"
#include "Int3x3.h"
#include <fstream>
#include <list>
#include <string>

/**
 * Possibilities for fcf-files:
 * - 
 */
class FCFfile: public mapcoeffile {
private:
    struct FCFcomponents {
        HKL hkl;
        double Imeas;
        double sigImeas;
        double Fcalc;
        double phicalc;
    };
    std::vector<FCFcomponents> fcfdata_;
    
    float a_,b_,c_;
    float alpha_, beta_, gamma_;
    double dhighres_;
    
    double weightWeaks_;
    
    std::string fa_filename_;
    std::vector<Int3x3> symops_;
    std::vector<std::string> symopsStrings_;
    
    void readsymop (std::ifstream& inp);
    void tosymop (const std::string& symstring, Int3x3& symop, int row) const;

    void read_fcf (const char* filename);
    void read_fcfheader (std::ifstream& inp);
    void read_fcfdata (std::ifstream& inp);

    std::vector<Reflex> read_fadata (const char* filename) const;
public:
    FCFfile(const char* filename, 
        const double downweight);
    FCFfile(const char* filename, 
        const char* fa_filename,
        const double downweight);
    ~FCFfile();
    
    std::vector<Reflex> reflections(int mapcoeff) const;
    
    float a() const { return a_; }
    float b() const { return b_; }
    float c() const { return c_; }
    float alpha() const { return alpha_; }
    float beta() const { return beta_; }
    float gamma() const { return gamma_; }
    float dhighres() const { return dhighres_; }
    
    std::vector<Int3x3> symops() const { return symops_; }
    std::vector<std::string> symopsStrings() const { return symopsStrings_; }
};

#endif	/* FCFFILE_H */

