/* 
 * File:   main.cpp
 * Author: tg
 *
 * Created on June 28, 2012, 10:24 PM
 */


#include <iostream>
#include <algorithm>

#include "phsfile.h"
#include "fcffile.h"
#include "myExcepts.h"
#include "Reflex.h"
#include "ReflUtils.h"
#include "HKL.h"
#include "ClipperMap.h"
#include "conversions.h"
#include "Options.h"
#include "usage.h"

#include <clipper/clipper.h>

/*
 * shelx2map
 * read in a shelx fcf-file or phs-file and convert it to a 
 * ccp4 map (for displaying in pymol etc.)
 * Automatic file type distinction between fcf and phs
 */
int main(int argc, char** argv) {
    clipper::Spacegroup sg;
    clipper::Cell cell;
    double dhighres(-1.0);
    double downweight(-1.0);
    std::vector<Reflex> reflections;
    std::string input, output;
    Options * myoptions(NULL);

    try {
        myoptions = new Options(argc, argv);
    } catch (const myExcepts::Conversion::Option& e) {
        delete myoptions;
        return EXIT_FAILURE;
    }
    if (myoptions->help()) {
        delete myoptions;
        return EXIT_SUCCESS;
    }

    input = myoptions->input();
    output = myoptions->output();
    downweight = myoptions->w();

    try {
        bool fom = myoptions->fom();
        Phsfile phsfile(input.c_str(), fom, downweight);
        std::string pdbfile = myoptions->pdbfile();
        if (pdbfile.empty()) {
            usage();
            std::cout << "**** Error: phs-file requires PDB file for"
                    << " spacegroup and cell (-s)\n";
            return -1;
        }
        syminfoFromPDB(pdbfile, sg, cell);
        reflections = phsfile.reflections();
        if (output.empty()) {
            output = phsfile.filename();
            output = mapfilename(output);
        }
        dhighres = ReflUtils::dmin(reflections, cell);
        if (dhighres < 0.0) {
            std::cerr << "*** Programming error: High resolution limit not set.\n";
            return -1;
        }
    }    
    catch (const myExcepts::Format::PHS& e) {
        int mapcoeff = myoptions->m();
        
        FCFfile map(input.c_str(), downweight);

        sg = clipperSG(map.symopsStrings());
        cell = clipperCell(map.a(), map.b(), map.c(),
                map.alpha(), map.beta(), map.gamma());
        dhighres = map.dhighres();
        if (dhighres < 0.0) {
            std::cerr << "*** Programming error: High resolution limit not set.\n";
            return -1;
        }

        if (mapcoeff < 0) /* m unset: creating diff map separately */ {
            reflections = map.reflections(0);
            if (output.empty()) {
                output = map.filename();
            }
            output = mapfilename(output, "_diff.map");

            ClipperMap clipperMap(reflections, sg, cell, dhighres);
            clipperMap.makeMap();
            clipperMap.writeCCP4map(output.c_str());

            /* prepare for second map (only if mapcoeff unset) */
            mapcoeff = 3;
            output = myoptions->output();

        }
        reflections = map.reflections(mapcoeff);
        if (output.empty()) {
            output = map.filename();
            if (mapcoeff == 0) {
                output = mapfilename(output, "_diff.map");
            } 
            else {
                std::ostringstream minfo;
                minfo << "_" << mapcoeff << "Fo-" << (mapcoeff-1)
                        << "Fc.map";
                output = mapfilename(output, minfo.str());
            }
        }

        if (output.empty()) {
            output = map.filename();
        }
    }    
    catch (...) {
        std::cerr << "***** Programming error\n";
        return -1;
    }

    /*
     * unnecessary steps with clipper:
     * stdsettings (reflections)
     * sort (reflections)
     * merge (reflections)
     */
    ClipperMap clipperMap(reflections, sg, cell, dhighres);
    clipperMap.makeMap();
    clipperMap.writeCCP4map(output.c_str());


    return 0;
}

