/* 
 * File:   myExcepts.h
 * Author: tg
 *
 * Created on July 10, 2012, 1:07 PM
 */

#ifndef MYEXCEPTS_H
#define	MYEXCEPTS_H

#include <sstream>

namespace myExcepts {
    class myExcepts {
    protected:
        const char* msg_;
    public:
        myExcepts (const char* text): msg_(text) {}
        ~myExcepts() throw() {}
        virtual const char* what() const throw() =0;
    };
    
    class Conceptual: public myExcepts {
    public:
        Conceptual (const char* text): myExcepts(text) {
        }

        const char* what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: Programming error - "
                    << msg_ << "\n";
            return msg.str().c_str();
        }
    };
    namespace Format {
        namespace FCF {
            class ListCode: public myExcepts {
            public:
                ListCode (const char* text): myExcepts(text) {}
                const char* what() const throw () {
                    std::ostringstream msg;
                    msg << " *** Error: fcf-file not LIST code 6\n"
                            << "       " << msg_ << "\n";
                    return msg.str().c_str();
                }
            };
        }
        class PHS: public myExcepts {
        public:
            PHS (): myExcepts ("") {}
            PHS (const char* text): myExcepts(text) {}
                const char* what() const throw () {
                    const std::string msg(" *** Error: Unable to read PHS file");
                    return msg.c_str();
                }
        };
        class FA: public myExcepts {
        public:
            FA (): myExcepts ("") {}
            FA (const char* text): myExcepts(text) {}
                const char* what() const throw () {
                    const std::string msg(" *** Error: Unable to read FA file");
                    return msg.c_str();
                }
        };
        class Symop: public myExcepts {
            public:
                Symop (const char* text): myExcepts(text) {}
                const char* what() const throw () {
                    std::ostringstream msg;
                    msg << " *** Error: The string \""
                            << msg_
                            << "\" cannot be interpreted as Symmetry operator\n";
                    return msg.str().c_str();
                }
        };
    }
    namespace IO {

        class Filename : public myExcepts {
        public:
            Filename(const char* text) : myExcepts(text) {
            }

            const char* what() const throw () {
                std::ostringstream msg;
                msg << " *** Error: Unabel to open file \""
                        << msg_
                        << "\"\n";
                return msg.str().c_str();
            }
        };
    }
    namespace Conversion {
        class Option: public myExcepts {
        public:
            Option(const char* text): myExcepts(text) {}
            const char* what() const throw () {
                std::ostringstream msg;
                msg << " *** Improper format of Input Option \""
                        << msg_
                        << "\"\n";
                return msg.str().c_str();
                
            }
        };
    }
}

#endif	/* MYEXCEPTS_H */

