/* 
 * File:   phsfile.cpp
 * Author: tg
 * 
 * Created on June 28, 2012, 10:30 PM
 */

#include "phsfile.h"
#include "myExcepts.h"

#include <fstream>
#include <iostream>
#include <cmath>

Phsfile::Phsfile(const char* filename, const bool fomweight, const double weight)
        : mapcoeffile(filename) {
    std::ifstream inp (filename);
    int h,k,l;
    double F, fom, phi, sigF;
    
    if (! inp.is_open()) {
        std::string fn (filename);
        fn += ".phs";
        inp.open(fn.c_str());
        if (! inp.is_open()) {
            std::string emsg ("Could not open ");
            emsg += fn;
            throw myExcepts::Format::PHS (emsg.c_str());
        }
    }
    /* update filename */
    this->filename(filename);
    
    while (true) {
        inp >> h >> k >> l
                >> F
                >> fom
                >> phi
                >> sigF;
        if (inp.fail()) {
            if (!inp.eof()) {
                throw myExcepts::Format::PHS ("not a phs file");
            }
            break;
        }
            
        HKL hkl(h,k,l);
        
        if (fomweight && weight < 0.0) {
            F *= fom;
        }
        if (weight > 0.0) {
            double scale = sigF/F;
            scale = 1.0 /
                (1.0 + weight * std::pow(scale, 4));
            F *= scale;

        }
        Reflex reflex(hkl, F, phi);
        reflections_.push_back(reflex);
    }
    std::cout << "---> PHS file read in.\n";
    std::cout << "     Total number of reflections: " << reflections_.size() << "\n";
    if (fomweight && weight < 0.0) {
        std::cout << "     Amplitudes weighted with FOM\n";
    }
    else if (weight >= 0.0) {
        std::cout << "     Amplitudes weighted with "
                << weight << "/(1+(sigF/F)^4)\n";
    }
    else if (! fomweight) {
        std::cout << "     Unweighted amplitudes\n";
    }
    
    inp.close();
}

Phsfile::~Phsfile() {
}

